/**
 * Module dependencies.
 */

var parser = require("../parser").parser;
var nodes = require('../nodes');
var fs = require('fs');
var http = require('http');
var path = require('path');
var sys = require('sys');
var spawn = require('child_process').spawn;
var style = require('../colors');
var utils = require('../utils');
var _ = require('underscore')._;
var async = require('async');
var findit = require('findit');

/**
 * Gherkin Parser.
 */

parser.yy = nodes;

var parseEnvFile = exports.parseEnvFile = function(callback) {
  if (__peanut__.envExists) {
    try {
      var env = __peanut__.env = require(process.cwd() +
                                 '/features/support/env');
      if (env.env && env.file && env.host && env.port) {
        setupTestServer(env, callback)
      } else { callback(null); }
    } catch(err) { callback(err); }
  } else {
    callback(null);
  }
};

function setupTestServer (env, callback) {
  process.env['NODE_ENV'] = env.env;
  var testServer = spawn('node', [env.file], { cwd: process.cwd() });

  var interval = setInterval(function() {
    http.get({ host: env.host, port: env.port, path: '/' }, function(res) {
      callback(null);
      clearInterval(interval);
    }).on('error', function(e) {});
  }, 100);
};

var parseStepDefinitions = exports.parseStepDefinitions = function() {
  try {
    _(findit.findSync('features')).each(function(file) {
      if (file.match(/\_steps.js$/) !== null) {
        var correctFilePath = process.cwd() + '/' + file;
        delete require.cache[path.resolve(correctFilePath)];
        require(correctFilePath);
      }
    });
    __peanut__.stepDefinitions = _(__peanut__.stepDefinitions).chain()
      .flatten().unique().value();
  } catch(err) { sys.puts(style.red(err)); }
};

var parseFeatures = exports.parseFeatures = function(callback) {
  async.map(__peanut__.options.path, statPath, function(err, results) {
    var fsStats = _.compact(results);

    async.map(fsStats, processFileOrDir, function(err, results) {
      var results = _(results).chain().flatten().compact().value();
      callback(err, results);
    });

    function processFileOrDir(stats, cb) {
      if (stats.isDirectory()) {
        var files = findit.sync(stats.path);
        async.map(files, parseFeature, function(err, results) {
          cb(err, results);
        });
      };

      if (stats.isFile()) {
        parseFeature(stats.path, cb);
      };
    };
  });

  function statPath(path, cb) {
    fs.stat(path, function(err, stats) {
      if (err) {
        cb(err, stats)
      } else {
        stats.path = path;
        cb(null, stats);
      }
    });
  };
};

function parseFeature(file, callback) {
  if (file.match(/\.feature$/) !== null) {
    fs.readFile(file, 'utf8', function (err, data) {
      if (err) throw err;
      parser.yy.file = new nodes.File();
      var parsedFile = parser.parse(data);
      parsedFile.featureFile = file;

      __peanut__.stepTotal += addStepCounts(parsedFile.feature);
      callback(err, parsedFile)
    });
  } else {
    callback(null);
  }
};

function addStepCounts(feature) {
  var stepCounts = 0;

  _.each(feature.scenarios, function(scenario) {
    var modifier = utils.nil(scenario.examples)
      ? 1 : scenario.examples.length > 0 ? scenario.examples.length : 1;
    stepCounts += scenario.steps.length * modifier;

    if (feature.background) {
      stepCounts += feature.background.steps.length * modifier;
    };
  });

  return stepCounts;
};
